@@ -38,8 +38,14 @@
 #include <libedataserverui/e-book-auth-util.h>
 #include <libedataserver/e-sexp.h>
 
+#include <libedataserverui/e-data-server-ui-marshal.h>
 #include "e-name-selector-entry.h"
 
+enum {
+	UPDATED,
+	LAST_SIGNAL
+};
+static guint signals[LAST_SIGNAL] = { 0 };
 #define ENS_DEBUG(x)
 
 #define COMPLETION_CUE_MIN_LEN 3
@@ -120,6 +126,12 @@ e_name_selector_entry_finalize (GObject *object)
 		G_OBJECT_CLASS (e_name_selector_entry_parent_class)->finalize (object);
 }
 
+static void
+e_name_selector_entry_updated (ENameSelectorEntry *entry, char *email)
+{
+	g_return_if_fail (E_IS_NAME_SELECTOR_ENTRY (entry));
+}
+
 static void
 e_name_selector_entry_class_init (ENameSelectorEntryClass *name_selector_entry_class)
 {
@@ -130,12 +142,22 @@ e_name_selector_entry_class_init (ENameSelectorEntryClass *name_selector_entry_c
 	object_class->set_property = e_name_selector_entry_set_property;
 	object_class->dispose      = e_name_selector_entry_dispose;
 	object_class->finalize     = e_name_selector_entry_finalize;
+	name_selector_entry_class->updated	   = e_name_selector_entry_updated;
 
 	widget_class->realize      = e_name_selector_entry_realize;
 
 	/* Install properties */
 
 	/* Install signals */
+	
+	signals[UPDATED] = g_signal_new ("updated",
+					 E_TYPE_NAME_SELECTOR_ENTRY,
+					 G_SIGNAL_RUN_FIRST,
+					 G_STRUCT_OFFSET (ENameSelectorEntryClass, updated),
+					 NULL,
+					 NULL,
+					 e_data_server_ui_marshal_VOID__POINTER,
+					 G_TYPE_NONE, 1, G_TYPE_POINTER);
 
 	g_type_class_add_private (object_class, sizeof(ENameSelectorEntryPrivate));
 }
@@ -1111,7 +1133,7 @@ user_insert_text (ENameSelectorEntry *name_selector_entry, gchar *new_text,
 		chars_inserted++;
 	}
 
-	if (chars_inserted == 1) {
+	if (chars_inserted >= 1) {
 		/* If the user inserted one character, kick off completion */
 		if (!name_selector_entry->update_completions_cb_id) {
 			name_selector_entry->update_completions_cb_id =
@@ -1124,10 +1146,6 @@ user_insert_text (ENameSelectorEntry *name_selector_entry, gchar *new_text,
 				g_idle_add ((GSourceFunc) type_ahead_complete_on_idle_cb,
 					    name_selector_entry);
 		}
-	} else if (chars_inserted > 1 && name_selector_entry->type_ahead_complete_cb_id) {
-		/* If the user inserted more than one character, prevent completion */
-		g_source_remove (name_selector_entry->type_ahead_complete_cb_id);
-		name_selector_entry->type_ahead_complete_cb_id = 0;
 	}
 
 	g_signal_handlers_unblock_by_func (name_selector_entry, user_delete_text, name_selector_entry);
@@ -1157,11 +1175,12 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 	if (gtk_editable_get_selection_bounds (GTK_EDITABLE (name_selector_entry), 
 					       &selection_start, 
 					       &selection_end)) 
-		if (g_utf8_get_char (g_utf8_offset_to_pointer (text, selection_end - 1)) == ',') 
+		if ((g_utf8_get_char (g_utf8_offset_to_pointer (text, selection_end)) == 0) ||
+		    (g_utf8_get_char (g_utf8_offset_to_pointer (text, selection_end)) == ","))
 			already_selected = TRUE;
 	
 	get_utf8_string_context (text, start_pos, str_context, 2);
-	get_utf8_string_context (text, end_pos-1, str_b_context, 2);
+	get_utf8_string_context (text, end_pos, str_b_context, 2);
 
 	g_signal_handlers_block_by_func (name_selector_entry, user_delete_text, name_selector_entry);
 
@@ -1182,8 +1201,13 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 	/* If the user is trying to delete a ','-character, we assume the user 
 	 * wants to remove the entire destination.
 	 */
-
-	if (((str_b_context [0] == ',' && str_b_context [1] == ' ') || str_b_context [1] == ',') && !already_selected) {
+#if 0	
+	printf("%d %d %d\n", already_selected, selection_start, selection_end);
+	printf("pos %d %d\n", start_pos, end_pos);
+	printf("it is -%c- -%c- -%c- -%c-\n", str_context[0], str_context[1], str_b_context[0], str_b_context[1]);
+	printf("it is -%d- -%d- -%d- -%d-\n", str_context[0], str_context[1], str_b_context[0], str_b_context[1]);
+#endif	
+	if ((str_b_context [0] != ',') && 0 && !already_selected && (str_b_context[1] == ' ' || str_b_context[1] == 0)) {
 
 		EDestination *dest = find_destination_at_position (name_selector_entry, end_pos-1);
 	  	const char *email = e_destination_get_email (dest);
@@ -1193,7 +1217,7 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 		 	 * Deleting this selection afterwards will leave the destination
 		 	 * empty. */
 	 
-			gint t = (str_b_context [1]==',')?end_pos:end_pos-1, b=t;
+			gint t = end_pos, b=t;
 			do {
 				t--;
 			} while (t >= 1 && text[t-1] != ',');
@@ -1298,6 +1322,7 @@ completion_match_selected (ENameSelectorEntry *name_selector_entry, GtkTreeModel
 	/* Place cursor at end of address */
 
 	gtk_editable_set_position (GTK_EDITABLE (name_selector_entry), cursor_pos);
+	g_signal_emit (name_selector_entry, signals[UPDATED], 0, destination, NULL);
 	return TRUE;
 }
 
@@ -1306,21 +1331,42 @@ entry_activate (ENameSelectorEntry *name_selector_entry)
 {
 	gint         cursor_pos;
 	gint         range_start, range_end;
-	const gchar *text;
 	ENameSelectorEntryPrivate *priv;
+	EContact      *contact;
+	EContactField  matched_field;
+	EDestination  *destination;
+	gchar         *textrep;
+	gint           range_len;
+	const gchar   *text;
+	gchar         *cue_str;
+	
+	cursor_pos = gtk_editable_get_position (GTK_EDITABLE (name_selector_entry));
+	if (cursor_pos < 0)
+		return;
 
 	priv = E_NAME_SELECTOR_ENTRY_GET_PRIVATE (name_selector_entry);
 
-	/* Show us what's really there */
+	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
+	get_range_at_position (text, cursor_pos, &range_start, &range_end);
+	range_len = range_end - range_start;
+	if (range_len < COMPLETION_CUE_MIN_LEN)
+		return;
 
-	cursor_pos = gtk_editable_get_position (GTK_EDITABLE (name_selector_entry));
+	destination = find_destination_at_position (name_selector_entry, cursor_pos);
+
+	cue_str = get_entry_substring (name_selector_entry, range_start, range_end);
+	if (!find_existing_completion (name_selector_entry, cue_str, &contact,
+				       &textrep, &matched_field)) {
+		g_free (cue_str);
+		return;
+	}
+	g_free (cue_str);	
 	sync_destination_at_position (name_selector_entry, cursor_pos, &cursor_pos);
 
 	/* Place cursor at end of address */
 
-	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
-	get_range_at_position (text, cursor_pos, &range_start, &range_end);
 	gtk_editable_set_position (GTK_EDITABLE (name_selector_entry), range_end);
+	g_signal_emit (name_selector_entry, signals[UPDATED], 0, destination, NULL);
 	
 	priv->is_completing = FALSE;
 }
@@ -1805,6 +1851,7 @@ static void
 popup_activate_inline_expand (ENameSelectorEntry *name_selector_entry, GtkWidget *menu_item)
 {
 	const char *email_list, *text;
+	gchar *sanitized_text;
 	EDestination *destination = name_selector_entry->popup_destination;
 	int position, start, end;
 
@@ -1818,8 +1865,9 @@ popup_activate_inline_expand (ENameSelectorEntry *name_selector_entry, GtkWidget
 
 	gtk_editable_delete_text (GTK_EDITABLE (name_selector_entry), start, end);
 
-	text = sanitize_string (e_destination_get_textrep (destination, FALSE));
-	gtk_editable_insert_text (GTK_EDITABLE (name_selector_entry), email_list, -1, &start);
+	sanitized_text = sanitize_string (email_list);
+	gtk_editable_insert_text (GTK_EDITABLE (name_selector_entry), sanitized_text, -1, &start);
+	g_free (sanitized_text);
 
 	g_signal_handlers_unblock_by_func (name_selector_entry, user_delete_text, name_selector_entry);
 
